#  Copyright (C) 2025 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.


from gvsbuild.utils.base_project import Project, ProjectType


def test_tools_registered():
    Project.add_all()
    tools = [p for p in Project._projects if p.type == ProjectType.TOOL]

    tool_names = [t.name for t in tools]
    assert "cmake" in tool_names
    assert "meson" in tool_names
    assert "ninja" in tool_names
    assert "perl" in tool_names
    assert "nasm" in tool_names
    assert "cargo" in tool_names
    assert "go" in tool_names
    assert "msys2" in tool_names


def test_cmake_tool():
    from gvsbuild.tools import ToolCmake

    cmake = ToolCmake()
    assert cmake.name == "cmake"
    assert cmake.version is not None
    assert cmake.archive_url is not None


def test_meson_tool():
    from gvsbuild.tools import ToolMeson

    meson = ToolMeson()
    assert meson.name == "meson"
    assert meson.version is not None
    assert meson.exe_name == "meson.py"


def test_ninja_tool():
    from gvsbuild.tools import ToolNinja

    ninja = ToolNinja()
    assert ninja.name == "ninja"
    assert ninja.version is not None
    assert ninja.exe_name == "ninja.exe"


def test_perl_tool():
    from gvsbuild.tools import ToolPerl

    perl = ToolPerl()
    assert perl.name == "perl"
    assert perl.version is not None


def test_nasm_tool():
    from gvsbuild.tools import ToolNasm

    nasm = ToolNasm()
    assert nasm.name == "nasm"
    assert nasm.version is not None
    assert nasm.exe_name == "nasm.exe"


def test_cargo_tool():
    from gvsbuild.tools import ToolCargo

    cargo = ToolCargo()
    assert cargo.name == "cargo"
    assert cargo.version is not None
    assert cargo.exe_name == "cargo.exe"


def test_go_tool():
    from gvsbuild.tools import ToolGo

    go = ToolGo()
    assert go.name == "go"
    assert go.version is not None


def test_msys2_tool():
    from gvsbuild.tools import ToolMsys2

    msys2 = ToolMsys2()
    assert msys2.name == "msys2"
    assert msys2.internal is True


def test_cmake_load_defaults(mock_opts):
    from gvsbuild.tools import ToolCmake

    cmake = ToolCmake()
    mock_opts.tools_root_dir = "C:\\tools"
    cmake.opts = mock_opts

    cmake.load_defaults()
    assert cmake.full_exe is not None
    assert cmake.tool_path is not None


def test_perl_load_defaults(mock_opts):
    from gvsbuild.tools import ToolPerl

    perl = ToolPerl()
    mock_opts.tools_root_dir = "C:\\tools"
    perl.opts = mock_opts

    perl.load_defaults()
    assert perl.full_exe is not None
    assert perl.tool_path is not None
    assert perl.base_dir is not None


def test_perl_get_base_dir(mock_opts):
    from gvsbuild.tools import ToolPerl

    perl = ToolPerl()
    mock_opts.tools_root_dir = "C:\\tools"
    perl.opts = mock_opts
    perl.load_defaults()

    base_dir = perl.get_base_dir()
    assert base_dir is not None


def test_msys2_get_path(mock_opts):
    from gvsbuild.tools import ToolMsys2

    msys2 = ToolMsys2()
    mock_opts.msys_dir = "C:\\msys64"
    mock_opts.tools_root_dir = "C:\\tools"
    msys2.opts = mock_opts
    msys2.load_defaults()

    result = msys2.get_path()
    assert result is not None
    assert isinstance(result, tuple)
    assert result[0] is None


def test_cargo_load_defaults(mock_opts):
    from gvsbuild.tools import ToolCargo

    cargo = ToolCargo()
    mock_opts.tools_root_dir = "C:\\tools"
    cargo.opts = mock_opts
    cargo.build_dir = "C:\\tools\\cargo"

    cargo.load_defaults()
    assert cargo.full_exe is not None
    if hasattr(cargo, "extra_env"):
        assert len(cargo.extra_env) > 0



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import project_add


@project_add
class Glibmm(Tarball, Meson):
    def __init__(self):
        Meson.__init__(
            self,
            "glibmm",
            prj_dir="glibmm",
            version="2.86.0",
            lastversion_even=True,
            repository="https://gitlab.gnome.org/GNOME/glibmm",
            archive_url="https://download.gnome.org/sources/glibmm/{major}.{minor}/glibmm-{version}.tar.xz",
            hash="39c0e9f6da046d679390774efdb9ad564436236736dc2f7825e614b2d4087826",
            dependencies=[
                "meson",
                "ninja",
                "libsigc++",
                "glib",
            ],
        )

    def build(self):
        Meson.build(
            self,
            meson_params="-Dbuild-examples=false -Dbuild-documentation=false",
        )

        self.install(r".\COPYING share\doc\glibmm")


@project_add
class Glibmm2_4(Tarball, Meson):
    def __init__(self):
        Meson.__init__(
            self,
            "glibmm-2.4",
            prj_dir="glibmm-2.4",
            version="2.66.8",
            lastversion_even=True,
            outdated_skip=True,
            repository="https://gitlab.gnome.org/GNOME/glibmm",
            archive_url="https://download.gnome.org/sources/glibmm/{major}.{minor}/glibmm-{version}.tar.xz",
            hash="64f11d3b95a24e2a8d4166ecff518730f79ecc27222ef41faf7c7e0340fc9329",
            dependencies=["meson", "ninja", "libsigc++-2.0", "glib"],
        )

    def build(self):
        Meson.build(
            self, meson_params="-Dbuild-examples=false -Dbuild-documentation=false"
        )

        self.install(r".\COPYING share\doc\glibmm-2.4")



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.


def test_outdated(app, runner):
    result = runner.invoke(app, ["outdated", "--help"])
    assert result.exit_code == 0
    assert "Check for outdated project versions" in result.stdout


def test_extra_arg(app, runner):
    result = runner.invoke(app, ["outdated", "extra"])
    # Cyclopts returns exit code 1 for errors
    assert result.exit_code in [1, 2]
    full_output = result.stdout + result.stderr
    assert (
        "extra" in full_output.lower()
        or "unexpected" in full_output.lower()
        or "unrecognized" in full_output.lower()
    )



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

import os

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class GStreamer(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gstreamer",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.28.1",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gstreamer/gstreamer-{version}.tar.xz",
            hash="b65e2ffa35bdbf8798cb75c23ffc3d05e484e48346ff7546844ba85217664504",
            dependencies=["meson", "ninja", "glib", "orc"],
            patches=[],
        )

        if self.opts.enable_gi:
            self.add_dependency("gobject-introspection")
        enable_gi = "enabled" if self.opts.enable_gi else "disabled"
        self.add_param(f"-Dintrospection={enable_gi}")

    def build(self):
        add_path = os.path.join(self.builder.opts.msys_dir, "usr", "bin")

        Meson.build(
            self, add_path=add_path, meson_params="-Dtests=disabled -Dexamples=disabled"
        )
        self.install(r".\COPYING share\doc\gstreamer")


@project_add
class Orc(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "orc",
            version="0.4.42",
            lastversion_even=True,
            repository="https://gitlab.freedesktop.org/gstreamer/orc",
            archive_url="https://gstreamer.freedesktop.org/src/orc/orc-{version}.tar.xz",
            hash="7ec912ab59af3cc97874c456a56a8ae1eec520c385ec447e8a102b2bd122c90c",
            dependencies=["meson", "ninja"],
        )

    def build(self):
        Meson.build(self, meson_params="-Dbenchmarks=disabled -Dtools=enabled")
        self.install(r"COPYING share\doc\orc")


@project_add
class GstPluginsBase(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-plugins-base",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.28.1",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gst-plugins-base/gst-plugins-base-{version}.tar.xz",
            hash="1446a4c2a92ff5d78d88e85a599f0038441d53333236f0c72d72f21a9c132497",
            dependencies=[
                "meson",
                "ninja",
                "gstreamer",
                "opus",
                "ogg",
                "graphene",
                "pango",
            ],
        )
        # Examples depend on GTK3
        self.add_param("-Dexamples=disabled")

        if self.opts.enable_gi:
            self.add_dependency("gobject-introspection")
        enable_gi = "enabled" if self.opts.enable_gi else "disabled"
        self.add_param(f"-Dintrospection={enable_gi}")

    def build(self):
        Meson.build(
            self, meson_params=f'-Dc_link_args="{self.builder.gtk_dir}\\lib\\ogg.lib"'
        )
        self.install(r".\COPYING share\doc\gst-plugins-base")


@project_add
class GstPluginsGood(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-plugins-good",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.28.1",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gst-plugins-good/gst-plugins-good-{version}.tar.xz",
            hash="738e26aee41b7a62050e40b81adc017a110a7f32d1ec49fa6a0300846c44368d",
            dependencies=[
                "meson",
                "ninja",
                "gst-plugins-base",
                "libvpx",
            ],
        )

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gst-plugins-good")


@project_add
class GstPluginsBad(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-plugins-bad",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.28.1",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gst-plugins-bad/gst-plugins-bad-{version}.tar.xz",
            hash="56c1593787f8b5550893d59e4ff29e6bcccf34973316fa55e34ce493e04313a2",
            dependencies=[
                "meson",
                "ninja",
                "gst-plugins-base",
                "libnice",
                "webrtc-audio-processing",
                "openssl",
                "libsrtp2",
            ],
            patches=[
                "wasapisink-reduce-buffer-latency.patch",
            ],
        )
        self.add_param("-Dcurl=disabled")
        self.add_param("-Dcurl-ssh2=disabled")

        # Adding webrtc-audio-processing adds the isac plugin. It fails to compile.
        self.add_param("-Disac=disabled")

        if self.opts.enable_gi:
            self.add_dependency("gobject-introspection")
        enable_gi = "enabled" if self.opts.enable_gi else "disabled"
        self.add_param(f"-Dintrospection={enable_gi}")

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gst-plugins-bad")


@project_add
class GstPluginsUgly(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-plugins-ugly",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.28.1",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gst-plugins-ugly/gst-plugins-ugly-{version}.tar.xz",
            hash="4082f3cb063fccc3ffc04e5ab0854bafde82d1b373eb3c9eaa28115dd3f95a78",
            dependencies=["meson", "ninja", "gst-plugins-base"],
        )

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gst-plugins-ugly")


@project_add
class GstDevTools(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-devtools",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.28.1",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gst-devtools/gst-devtools-{version}.tar.xz",
            hash="a4e49cd082972a132ca5f54be52a3c386db37c4cb0e487e017ba00d83a5f985d",
            dependencies=[
                "meson",
                "ninja",
                "json-glib",
                "gst-plugins-base",
                "gst-plugins-bad",
                "gst-rtsp-server",
            ],
        )

        if self.opts.enable_gi:
            self.add_dependency("gobject-introspection")
        enable_gi = "enabled" if self.opts.enable_gi else "disabled"
        self.add_param(f"-Dintrospection={enable_gi}")

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gst-devtools")


@project_add
class GstPython(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-python",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.28.1",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gst-python/gst-python-{version}.tar.xz",
            hash="d47cea95adb95ba10443ed7812c7c5fa0807aef43b98cd1c6d8fb9f9a86f7085",
            dependencies=[
                "meson",
                "ninja",
                "pygobject",
                "gst-plugins-base",
                "gst-plugins-bad",
                "gst-rtsp-server",
            ],
        )

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gst-python")


@project_add
class GstLibav(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-libav",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.28.1",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gst-libav/gst-libav-{version}.tar.xz",
            hash="bfa91aaca38d0fd8addcdd559e35b7541e3f32a5f410194ec4ba18040defee9b",
            # TODO try remove gst-plugins-base
            dependencies=["meson", "ninja", "pygobject", "ffmpeg", "gst-plugins-base"],
        )

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gst-libav")


@project_add
class GstRtspServer(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-rtsp-server",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.28.1",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gst-rtsp-server/gst-rtsp-server-{version}.tar.xz",
            hash="6cd0e86af20cfb749bdc24d2f5c30f4e60a3e8d198ae4e02e754b9937bdbf832",
            dependencies=["meson", "ninja", "gstreamer"],
        )

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gst-rtsp-server")


@project_add
class GstPluginGtk4(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-plugin-gtk4",
            version="0.14.3",
            repository="https://gitlab.freedesktop.org/gstreamer/gst-plugins-rs",
            archive_url="https://gitlab.freedesktop.org/gstreamer/gst-plugins-rs/-/archive/{version}/gst-plugins-rs-{version}.tar.gz?ref_type=tags",
            archive_filename="gst-plugins-rs-{version}.tar.gz",
            hash="0609b2459ec29f4678edd98e69b6b0a473ef4a303d60645260245dbc23f75167",
            dependencies=[
                "meson",
                "cargo",
                "gst-plugins-base",
                "gst-plugins-bad",
                "gtk4",
            ],
        )
        self.add_param("-Dgtk4=enabled")
        self.add_param("--auto-features=disabled")

    def build(self):
        self.builder.exec_cargo("install cargo-c --locked")
        Meson.build(self)
        self.install(r".\LICENSE-MPL-2.0 share\doc\gst-plugin-gtk4")



#  Copyright (C) 2025 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

import os

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class DirectXHeaders(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "directx-headers",
            repository="https://github.com/microsoft/DirectX-Headers",
            version="1.619.1",
            archive_url="https://github.com/microsoft/DirectX-Headers/archive/refs/tags/v{version}.tar.gz",
            archive_filename="directx-headers-v{version}.tar.gz",
            hash="6193774904c940eebb9b0c51b816b93dd776cfeb25a951f0f4a58f22387e5008",
            dependencies=["meson", "ninja"],
            patches=[],
        )

    def build(self):
        Meson.build(self, meson_params="-Dbuild-test=false")
        self.install(r".\LICENSE share\doc\directx-headers")

    def post_install(self):
        lib_dir = os.path.join(self.builder.gtk_dir, "lib")
        self.builder.exec_msys(
            ["mv", "libd3dx12-format-properties.a", "d3dx12-format-properties.lib"],
            working_dir=lib_dir,
        )
        self.builder.exec_msys(
            ["mv", "libDirectX-Guids.a", "DirectX-Guids.lib"],
            working_dir=lib_dir,
        )



#  Copyright (C) 2025 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.
import gvsbuild.groups  # noqa: F401
import gvsbuild.projects  # noqa: F401
import gvsbuild.tools  # noqa: F401
from gvsbuild.deps import compute_deps, make_graph, print_deps
from gvsbuild.utils.base_project import Project


def setup_module():
    Project.add_all()


def test_deps_help(app, runner, console):
    result = runner.invoke(app, ["deps", "--help"], console=console)
    assert result.exit_code == 0
    assert "Show project dependencies" in result.output
    assert "flatten" in result.output.lower()
    assert "graph" in result.output.lower()


def test_deps_default(app, runner):
    result = runner.invoke(app, ["deps"])
    assert result.exit_code == 0
    assert "Projects dependencies:" in result.output


def test_deps_flatten(app, runner):
    result = runner.invoke(app, ["deps", "--flatten"])
    assert result.exit_code == 0
    assert ">" in result.output


def test_deps_with_tools(app, runner):
    result = runner.invoke(app, ["deps", "--dep-tools"])
    assert result.exit_code == 0


def test_make_graph(tmp_path):
    output_file = tmp_path / "test.gv"
    make_graph(out_file=str(output_file), skip=[])
    assert output_file.exists()
    content = output_file.read_text()
    assert "digraph gtk3dep" in content
    assert content.strip().endswith("};")


def test_make_graph_with_all(tmp_path):
    output_file = tmp_path / "test_all.gv"
    make_graph(out_file=str(output_file), put_all=True, skip=[])
    assert output_file.exists()
    content = output_file.read_text()
    assert "BUILD" in content or "digraph" in content


def test_make_graph_with_tools(tmp_path):
    output_file = tmp_path / "test_tools.gv"
    make_graph(out_file=str(output_file), add_tools=True, skip=[])
    assert output_file.exists()


def test_make_graph_with_groups(tmp_path):
    output_file = tmp_path / "test_groups.gv"
    make_graph(out_file=str(output_file), add_groups=True, skip=[])
    assert output_file.exists()


def test_make_graph_inverted(tmp_path):
    output_file = tmp_path / "test_inverted.gv"
    make_graph(out_file=str(output_file), invert_dep=True, skip=[])
    assert output_file.exists()


def test_make_graph_with_skip(tmp_path):
    output_file = tmp_path / "test_skip.gv"
    make_graph(out_file=str(output_file), skip=["zlib"])
    assert output_file.exists()
    content = output_file.read_text()
    assert '"zlib"' not in content


def test_deps_graph_command(app, runner, tmp_path):
    output_file = tmp_path / "test_command.gv"
    result = runner.invoke(
        app, ["deps", "--graph", "--gv-file", str(output_file), "--skip", ""]
    )
    assert output_file.exists() or result.exit_code in [0, 1]


def test_compute_deps(mock_project):
    compute_deps(mock_project)
    assert hasattr(mock_project, "all_dependencies")


def test_compute_deps_with_nested(mocker):
    dep2 = mocker.Mock(spec=["dependencies", "all_dependencies", "name"])
    dep2.dependencies = []
    dep2.all_dependencies = []
    dep2.name = "dep2"

    dep1 = mocker.Mock(spec=["dependencies", "all_dependencies", "name"])
    dep1.dependencies = [dep2]
    dep1.all_dependencies = [dep2]
    dep1.name = "dep1"

    main_proj = mocker.Mock(spec=["dependencies", "name"])
    main_proj.dependencies = [dep1]
    main_proj.name = "main"

    compute_deps(main_proj)
    assert hasattr(main_proj, "all_dependencies")
    assert len(main_proj.all_dependencies) > 0


def test_print_deps_basic(capsys):
    print_deps(flatten=False, add_all=False)
    captured = capsys.readouterr()
    assert "Projects dependencies:" in captured.out


def test_print_deps_flatten(capsys):
    print_deps(flatten=True, add_all=False)
    captured = capsys.readouterr()
    assert ">" in captured.out


def test_print_deps_add_all(capsys):
    print_deps(flatten=False, add_all=True)
    captured = capsys.readouterr()
    assert "Projects dependencies:" in captured.out



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import CmakeProject
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Libssh(Tarball, CmakeProject):
    def __init__(self):
        Project.__init__(
            self,
            "libssh",
            version="0.11.3",
            repository="https://git.libssh.org/projects/libssh.git",
            archive_url="https://www.libssh.org/files/{major}.{minor}/libssh-{version}.tar.xz",
            hash="7d8a1361bb094ec3f511964e78a5a4dba689b5986e112afabe4f4d0d6c6125c3",
            dependencies=["zlib", "openssl", "cmake", "ninja"],
        )

    def build(self):
        CmakeProject.build(self, cmake_params="-DWITH_ZLIB=ON", use_ninja=True)
        self.install(r".\COPYING share\doc\libssh")


@project_add
class Libssh2(Tarball, CmakeProject):
    def __init__(self):
        Project.__init__(
            self,
            "libssh2",
            version="1.11.1",
            repository="https://github.com/libssh2/libssh2",
            archive_url="https://www.libssh2.org/download/libssh2-{version}.tar.gz",
            hash="d9ec76cbe34db98eec3539fe2c899d26b0c837cb3eb466a56b0f109cabf658f7",
            dependencies=[
                "cmake",
                "ninja",
            ],
        )

    def build(self):
        CmakeProject.build(self, cmake_params="-DWITH_ZLIB=ON", use_ninja=True)
        self.install(r".\COPYING share\doc\libssh2")



#  Copyright (C) 2025 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Libsass(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "libsass",
            version="3.6.6",
            outdated_skip=True,
            archive_url="https://github.com/sass/libsass/archive/refs/tags/{version}.tar.gz",
            hash="11f0bb3709a4f20285507419d7618f3877a425c0131ea8df40fe6196129df15d",
            archive_filename="libsass-{version}.tar.gz",
            dependencies=["ninja", "meson"],
        )

    def build(self):
        Meson.build(self)
        self.install(r"LICENSE share\doc\libsass")


@project_add
class Sassc(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "sassc",
            version="3.6.2",
            archive_url="https://github.com/sass/sassc/archive/refs/tags/{version}.tar.gz",
            hash="608dc9002b45a91d11ed59e352469ecc05e4f58fc1259fc9a9f5b8f0f8348a03",
            archive_filename="sassc-{version}.tar.gz",
            dependencies=["ninja", "meson", "libsass"],
        )

    def build(self):
        Meson.build(self)
        self.install(r"LICENSE share\doc\sassc")



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.


"""Main build script."""

import rich
from cyclopts import App

import gvsbuild.groups  # noqa: F401
import gvsbuild.projects  # noqa: F401
import gvsbuild.tools  # noqa: F401
from gvsbuild.build import build
from gvsbuild.deps import deps
from gvsbuild.list import list_
from gvsbuild.outdated import outdated

rich.reconfigure(markup=False)

app = App(
    help="Build GTK for Windows",
    help_on_error=True,
)
app.command(build)
app.command(outdated)
app.command(name="list")(list_)
app.command(deps)

app.register_install_completion_command()


def run():
    app()



#  Copyright (C) 2026 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.
from subprocess import CalledProcessError

import pytest

from gvsbuild.utils.base_project import Options
from gvsbuild.utils.builder import Builder, CheckVsInstallError


def test_vs_check_error_if_version_not_matching():
    opts = Options()
    opts.vs_ver = "17"
    builder = Builder.__new__(Builder)
    builder.opts = opts
    builder.vs_ver_year = "vs2022"

    with pytest.raises(
        CheckVsInstallError, match="Doesn't match target Visual Studio version of 17"
    ):
        builder._Builder__check_vs_install(
            opts, r"C:\Program Files\Microsoft Visual Studio\18\Community", True
        )


def test_vs_check_error_if_not_vcvars_exists(tmp_path):
    opts = Options()
    opts.platform = "x64"
    builder = Builder.__new__(Builder)
    builder.opts = opts

    with pytest.raises(CheckVsInstallError, match="vcvars64.bat could not be found"):
        builder._Builder__check_vs_install(opts, str(tmp_path), False)


def test_vs_check_error_if_vcvars_fails(tmp_path):
    opts = Options()
    opts.platform = "x64"
    builder = Builder.__new__(Builder)
    builder.opts = opts
    vcvars_path = tmp_path / "VC" / "Auxiliary" / "Build" / "vcvars64.bat"
    vcvars_path.parent.mkdir(parents=True)
    vcvars_path.write_text("exit 1")

    with pytest.raises(CalledProcessError):
        builder._Builder__check_vs_install(opts, str(tmp_path), False)


def test_vs_check_error_if_vcvars_no_env(tmp_path):
    opts = Options()
    opts.platform = "x64"
    builder = Builder.__new__(Builder)
    builder.opts = opts
    vcvars_path = tmp_path / "VC" / "Auxiliary" / "Build" / "vcvars64.bat"
    vcvars_path.parent.mkdir(parents=True)
    vcvars_path.write_text("exit 0")

    with pytest.raises(
        CheckVsInstallError, match="did not export environment variables properly"
    ):
        builder._Builder__check_vs_install(opts, str(tmp_path), False)


def test_vs_check_success(tmp_path):
    opts = Options()
    opts.platform = "x64"
    builder = Builder.__new__(Builder)
    builder.opts = opts
    vcvars_path = tmp_path / "VC" / "Auxiliary" / "Build" / "vcvars64.bat"
    vcvars_path.parent.mkdir(parents=True)
    vcvars_path.write_text("SET FOO=BAR")
    builder._Builder__check_vs_install(opts, str(tmp_path), False)



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from enum import Enum
from pathlib import Path
from typing import Annotated

from cyclopts import Group, Parameter, ValidationError

from gvsbuild.utils.base_project import Options, Project, ProjectType
from gvsbuild.utils.builder import Builder
from gvsbuild.utils.simple_ui import log
from gvsbuild.utils.utils import ordered_set


def validate_ninja_opts(type_, value: str) -> None:
    """Validator for ninja_opts to ensure it starts with a dash."""
    if value and not (value.startswith("-") or value.startswith("--")):
        raise ValidationError(
            f"ninja-opts must start with a dash (- or --). Got: '{value}'. "
            "Examples: --ninja-opts -j2, --ninja-opts --verbose"
        )


BUILD_CONFIG_GROUP = Group("Build Configuration", sort_key=0)
DIRECTORY_GROUP = Group("Directory Options", sort_key=1)
VS_SDK_GROUP = Group("Visual Studio and SDK Options", sort_key=2)
NET_GROUP = Group(".NET Options", sort_key=3)
BUILD_OPTIONS_GROUP = Group("Options to Pass to Build Systems", sort_key=4)
SKIP_CLEANUP_GROUP = Group("Skip and Cleanup Options", sort_key=5)
ZIP_GROUP = Group("Zip Options", sort_key=6)
INTROSPECTION_GROUP = Group("Introspection Options", sort_key=7)
OPENSSL_GROUP = Group("OpenSSL Options", sort_key=8)
FFMPEG_GROUP = Group("FFmpeg Options", sort_key=9)
LOGGING_GROUP = Group("Logging Options", sort_key=10)
ENVIRONMENT_GROUP = Group("Environment Options", sort_key=11)


def __get_projects_to_build(opts):
    to_build = ordered_set()
    for name in opts.projects:
        if name == "all":
            for proj in Project.list_projects():
                if proj.type == ProjectType.PROJECT:
                    to_build.add(proj)
        p = Project.get_project(name)
        if opts.deps:
            for dep in p.all_dependencies:
                to_build.add(dep)
        to_build.add(p)
        if opts.clean_built:
            p.clean = True
        if name == "openssl" and opts.enable_fips:
            to_build.add(Project.get_project("openssl-fips"))

    # See if we need to drop some project
    if opts.skip:
        for s in opts.skip:
            if s not in Project.get_names():
                log.error_exit(
                    s
                    + " is not a valid project name, available projects are:\n\t"
                    + "\n\t".join(Project.get_names())
                )

            p = Project.get_project(s)
            if p in to_build:
                log.debug(f"Dropped project {s}")
                to_build.remove(p)
    return to_build


def __parse_extra_opts(extra_opts: list[str]) -> dict[str, list[str]]:
    if extra_opts is None:
        return {}
    parsed_opts = {}
    for eo in extra_opts:
        project, opts = eo.split(":")
        parsed_opts[project] = opts.split(";")
    return parsed_opts


class Platform(str, Enum):
    x64 = "x64"
    x86 = "x86"


class Configuration(str, Enum):
    debug = "debug"
    release = "release"
    debug_optimized = "debug-optimized"


class VsVer(str, Enum):
    vs2013 = "12"
    vs2015 = "14"
    vs2017 = "15"
    vs2019 = "16"
    vs2022 = "17"
    vs2026 = "18"


class WinSdkVersion(str, Enum):
    sdk_8 = "8.1"
    sdk_10150 = "10.0.10150.0"
    sdk_10240 = "10.0.10240.0"
    sdk_10586 = "10.0.10586.0"
    sdk_14393 = "10.0.14393.0"
    sdk_15063 = "10.0.15063.0"
    sdk_16299 = "10.0.16299.0"
    sdk_17134 = "10.0.17134.0"
    sdk_17763 = "10.0.17763.0"
    sdk_22621 = "10.0.22621.0"


def build(
    projects: list[str],
    /,
    *,
    platform: Annotated[Platform, Parameter(group=BUILD_CONFIG_GROUP)] = Platform.x64,
    configuration: Annotated[
        Configuration, Parameter(group=BUILD_CONFIG_GROUP)
    ] = Configuration.debug_optimized,
    check_hash: Annotated[bool, Parameter(group=BUILD_CONFIG_GROUP)] = False,
    build_dir: Annotated[Path, Parameter(group=DIRECTORY_GROUP)] = Path(
        r"C:\gtk-build"
    ),
    msys_dir: Annotated[Path | None, Parameter(group=DIRECTORY_GROUP)] = None,
    archives_download_dir: Annotated[
        Path | None, Parameter(group=DIRECTORY_GROUP)
    ] = None,
    export_dir: Annotated[Path | None, Parameter(group=DIRECTORY_GROUP)] = None,
    patches_root_dir: Annotated[Path | None, Parameter(group=DIRECTORY_GROUP)] = None,
    tools_root_dir: Annotated[Path | None, Parameter(group=DIRECTORY_GROUP)] = None,
    git_expand_dir: Annotated[Path | None, Parameter(group=DIRECTORY_GROUP)] = None,
    vs_ver: Annotated[VsVer, Parameter(group=VS_SDK_GROUP)] = VsVer.vs2022,
    vs_install_path: Annotated[Path | None, Parameter(group=VS_SDK_GROUP)] = None,
    win_sdk_ver: Annotated[WinSdkVersion | None, Parameter(group=VS_SDK_GROUP)] = None,
    net_target_framework: Annotated[str | None, Parameter(group=NET_GROUP)] = None,
    net_target_framework_version: Annotated[
        str | None, Parameter(group=NET_GROUP)
    ] = None,
    msbuild_opts: Annotated[str | None, Parameter(group=BUILD_OPTIONS_GROUP)] = None,
    ninja_opts: Annotated[
        str | None,
        Parameter(
            group=BUILD_OPTIONS_GROUP,
            validator=validate_ninja_opts,
            allow_leading_hyphen=True,
        ),
    ] = None,
    cargo_opts: Annotated[str | None, Parameter(group=BUILD_OPTIONS_GROUP)] = None,
    extra_opts: Annotated[
        list[str] | None, Parameter(group=BUILD_OPTIONS_GROUP)
    ] = None,
    clean: Annotated[bool, Parameter(group=SKIP_CLEANUP_GROUP)] = False,
    clean_built: Annotated[bool, Parameter(group=SKIP_CLEANUP_GROUP)] = False,
    deps: Annotated[bool, Parameter(group=SKIP_CLEANUP_GROUP)] = True,
    skip: Annotated[list[str] | None, Parameter(group=SKIP_CLEANUP_GROUP)] = None,
    from_scratch: Annotated[bool, Parameter(group=SKIP_CLEANUP_GROUP)] = False,
    keep_tools: Annotated[bool, Parameter(group=SKIP_CLEANUP_GROUP)] = False,
    fast_build: Annotated[bool, Parameter(group=SKIP_CLEANUP_GROUP)] = False,
    keep_going: Annotated[bool, Parameter(group=SKIP_CLEANUP_GROUP)] = False,
    use_env: Annotated[bool, Parameter(group=ENVIRONMENT_GROUP)] = False,
    make_zip: Annotated[bool, Parameter(group=ZIP_GROUP)] = False,
    zip_continue: Annotated[bool, Parameter(group=ZIP_GROUP)] = False,
    py_wheel: Annotated[bool, Parameter(group=INTROSPECTION_GROUP)] = False,
    enable_gi: Annotated[bool, Parameter(group=INTROSPECTION_GROUP)] = False,
    enable_fips: Annotated[bool, Parameter(group=OPENSSL_GROUP)] = False,
    ffmpeg_enable_gpl: Annotated[bool, Parameter(group=FFMPEG_GROUP)] = False,
    log_size: Annotated[int, Parameter(group=LOGGING_GROUP)] = 0,
    log_single: Annotated[bool, Parameter(group=LOGGING_GROUP)] = False,
    capture_out: Annotated[bool, Parameter(group=LOGGING_GROUP)] = False,
    verbose: Annotated[bool, Parameter(group=LOGGING_GROUP)] = False,
    debug: Annotated[bool, Parameter(group=LOGGING_GROUP)] = False,
    print_out: Annotated[bool, Parameter(group=LOGGING_GROUP)] = False,
):
    """Build a project or a list of projects.

    Examples:
    gvsbuild build libpng libffi
        Build libpng, libffi, and their dependencies (zlib).

    gvsbuild build --no-deps glib
        Build glib only.

    gvsbuild build --skip gtk4 --skip pycairo all
        Build everything except gtk4 and pycairo

    Args:
        projects: The project to build.
        platform: The platform to build for.
        configuration: The configuration to build for. "debug-optimized" only includes debug symbols for Meson and CMake projects - other projects' build tools will interpret the option as "release".
        build_dir: The full or relative path of the directory to build in.
        msys_dir: The directory of the msys installation. If not specified, automatically searches in common locations.
        archives_download_dir: The directory to download the source archives to. It will be created. If a source archive already exists here, it won't be downloaded again. Default is the build-dir\\src.
        export_dir: The directory to export the source archives to. It will be created. It creates an archive with the source code and any possible patch. Default is the build-dir\\export.
        patches_root_dir: The directory where you checked out https://github.com/wingtk/gvsbuild.git. Default is the build-dir\\github\\gvsbuild.
        tools_root_dir: The directory where to install the downloaded tools. Default is $(build-dir)\\tools.
        vs_ver: Visual Studio version 12 (vs2013), 14 (vs2015), 15 (vs2017), 16 (vs2019), 17 (vs2022), 18 (vs2026).
        vs_install_path: The directory where you installed Visual Studio. Default is 'C:\\Program Files (x86)\\Microsoft Visual Studio $(vs-ver).0' (for vs-ver <= 14) or 'C:\\Program Files (x86)\\Microsoft Visual Studio\\20xx (2017 for vs-ver 15, 2019 for vs-ver 16, ...). If not set, the script look automatically under Professional, BuildTools, Enterprise, Community, and Preview sub directory until it finds the startup batch file.
        win_sdk_ver: The Windows SDK version to use for building, used to initialize the Visual Studio build environment. It can be 8.1 (for windows 8 compatibility) or 10.0.xxxxx.0, where xxxxx, at the moment, can be 10150, 10240, 10586, 14393, 15063, 16299, 17134, or 17763 depending on the VS version / installation's options. If you don't specify one the scripts tries to locate the used one to pass the value to the msbuild command.
        net_target_framework: .net target framework. If set then TargetFrameworks parameter is passed down to msbuild with the specific target. i.e net45.
        net_target_framework_version: .net target framework version. If set then TargetFrameworkVersion parameter is passed down to msbuild with the specific version. i.e v4.6.2.
        check_hash: If set, only check the hash of the downloaded archives, no build.
        clean: If set, clean the build directory before building.
        clean_built: If set, clean only the projects asked on the command line, not all the ones to build (via a dependency).
        deps: If not set, don't build the dependencies of the projects.
        msbuild_opts: Command line options to pass to msbuild.
        skip: Project to avoid building, can be run multiple times.
        use_env: Use and keep the calling environment for LIB, LIBPATH, INCLUDE and PATH.
        make_zip: Create singles zips of the projects built under the build-dir\\dist\\vsXXXX[-sdkVer]\\[platform]\\[configuration], for example 'C:\\gtk-build\\dist\\vs2015-8.1\\win32\\release'. NOTE: the destination dir (e.g. 'C:\\gtk-build\\gtk\\win32\\release') will be cleared completely before the build!
        zip_continue: If set, don't initialize the zip creation phase and keep the destination dir.
        from_scratch: Start from scratch, deleting the build and the destination directories of the project for the current platform/configuration setup (e.g. 'C:\\gtk-build\\build\\x64\\release' and 'C:\\gtk-build\\gtk\\x64\\release' and the common tools ('C:\\gtk-build\\tools').
        keep_tools: Active only when used with --from-scratch, keep and don't delete the (common) tool directory.
        fast_build: Don't build a project if it's already built and not updated. Note: you can have wrong results if you change only the patches or the script (updating the tarball or the git source is handled correctly).
        keep_going: Continue the build even on errors, dropping the projects that depends on the failed ones.
        py_wheel: pycairo/pygobject: build also the wheel distribution format.
        enable_gi: For the GTK stack, create the .gir/.typelib files for gobject introspection.
        enable_fips: Build the FIPS validated cryptographic module.
        ffmpeg_enable_gpl: ffmpeg: build with the gpl libraries/modules.
        log_size: Maximum log size (in kilobytes) before restarting with a new file.
        log_single: If set, always start a new log file, with date and time.
        capture_out: If set, capture the output of the build commands and write it to the log file.
        verbose: If set, print the output of the build commands to the console.
        debug: If set, print debug messages to the console.
        print_out: With --capture-out active print the result of the commands also on stdout.
        ninja_opts: Command line options to pass to ninja, e.g. to limit the use (-j 2) or for debug purposes.
        cargo_opts: Command line options to pass to cargo.
        extra_opts: Additional command line options to pass to specific project. Example: --extra_opts <project>:<option1>[;<option1>...].
        git_expand_dir: The directory where the projects from git are expanded and updated.
    """
    opts = Options()
    opts.verbose = verbose
    opts.debug = debug
    if build_dir:
        build_dir = Path(build_dir).resolve()
    opts.build_dir = str(build_dir)
    log.configure(str(build_dir / "logs"), opts)
    opts.platform = platform.value
    opts.configuration = configuration.value
    if opts.configuration == Configuration.debug_optimized:
        # Some build systems take "opts.configuration" directly, and won't support our
        # custom "debug-optimized" string. Convert it back to the standard
        # "release" string but track that we still want debug symbols where possible.
        opts.configuration = Configuration.release.value
        opts.release_configuration_is_actually_debug_optimized = True
    log.message(f"Build type is {configuration}")
    if not archives_download_dir:
        archives_download_dir = build_dir / "src"
    opts.archives_download_dir = str(archives_download_dir)
    opts.export_dir = str(export_dir) if export_dir else str(build_dir / "export")
    if not patches_root_dir:
        patches_root_dir = Path(__file__).parent / "patches"
    opts.patches_root_dir = str(patches_root_dir)
    if tools_root_dir:
        opts.tools_root_dir = str(tools_root_dir)
    else:
        opts.tools_root_dir = str(build_dir / "tools")
    opts.vs_ver = vs_ver.value
    opts.vs_install_path = vs_install_path
    opts.win_sdk_ver = win_sdk_ver.value if win_sdk_ver else None
    if git_expand_dir:
        opts.git_expand_dir = str(git_expand_dir)
    else:
        opts.git_expand_dir = str(archives_download_dir / "git-exp")
    opts.net_target_framework = net_target_framework
    opts.net_target_framework_version = net_target_framework_version
    opts.msys_dir = msys_dir
    opts.clean = clean
    opts.msbuild_opts = msbuild_opts
    opts.use_env = use_env
    opts.deps = deps
    opts.check_hash = check_hash
    opts.skip = skip
    opts.make_zip = make_zip
    opts.zip_continue = zip_continue
    opts.from_scratch = from_scratch
    opts.keep_tools = keep_tools
    opts.fast_build = fast_build
    opts.keep_going = keep_going
    opts.clean_built = clean_built
    opts.py_wheel = py_wheel
    opts.enable_gi = enable_gi
    opts.enable_fips = enable_fips
    opts.ffmpeg_enable_gpl = ffmpeg_enable_gpl
    opts.log_size = log_size
    opts.log_single = log_single
    opts.cargo_opts = cargo_opts
    opts.ninja_opts = ninja_opts
    opts.extra_opts = __parse_extra_opts(extra_opts if extra_opts else [])
    opts.capture_out = capture_out
    opts.print_out = print_out

    if opts.make_zip and not opts.deps:
        log.error_exit("Options --make-zip and --no-deps are not compatible")
    prop_file = patches_root_dir / "stack.props"
    if not Path.is_file(prop_file):
        log.error_exit(
            f"Missing 'stack.props' file on directory {opts.patches_root_dir}.\n"
            "Wrong or missing --patches-root-dir option?"
        )

    opts.projects = projects
    Project.opts = opts
    # now add the tools/projects/groups
    Project.add_all()

    for p in opts.projects:
        if p not in Project.get_names():
            log.error_exit(
                p
                + " is not a valid project name, available projects are:\n\t"
                + "\n\t".join(Project.get_names())
            )

    if log.debug_on():
        log.debug("Options are:")
        for co in sorted(opts.__dict__.keys()):
            v = opts.__dict__[co]
            pv = f"'{v}'" if isinstance(v, str) else repr(v)
            log.message_indent(f"'{co}': {pv}, ")
    builder = Builder(opts)
    builder.preprocess()

    to_build = __get_projects_to_build(opts)
    if not to_build:
        log.error_exit("nothing to do")
    log.debug(f"building {[p.name for p in to_build]}")

    builder.build(to_build)
